@@ -1,16 +1,24 @@
+"""Contains amm related methods and data."""
+
 import json
-import pyperp.MetaData
+from pyperp import MetaData
 from pyperp.utils import estimatedFundingRate, formatUnits
+import pkgutil
 
-def getAmmInfo(provider,pair=None):
-    with open("abi/InsuranceFund.json") as f:
-        insuranceFundAbi = json.load(f)
-    with open("abi/ClearingHouse.json") as f:
-        clearingHouseAbi = json.load(f)
-    with open("abi/Amm.json") as f:
-        ammAbi = json.load(f)
 
-    meta = MetaData.MetaData(provider.testnet)
+def getAmmInfo(provider, pair=None):
+    """
+    Return Amm information for all amms or a particular amm.
+
+    Arguments:
+    providers -- a Providers object
+    pair -- A string value representing a pair. Choose from AvailableAmms in constants.py
+    """
+    insuranceFundAbi = json.loads(pkgutil.get_data(__name__, "abi/InsuranceFund.json"))
+    clearingHouseAbi = json.loads(pkgutil.get_data(__name__, "abi/ClearingHouse.json"))
+    ammAbi = json.loads(pkgutil.get_data(__name__, "abi/Amm.json"))
+
+    meta = MetaData(provider.testnet)
     insuranceFundAddr = meta.getL2ContractAddress("InsuranceFund")
     clearingHouseAddr = meta.getL2ContractAddress("ClearingHouse")
 
@@ -23,13 +31,12 @@ def getAmmInfo(provider,pair=None):
 
     for addr in ammAddressList:
         amm = provider.l2.eth.contract(address=addr, abi=ammAbi)
-        print(ammAbi)
-        priceFeedKey = amm.functions.priceFeedKey().call().decode("utf-8","ignore")
+        priceFeedKey = amm.functions.priceFeedKey().call().decode("utf-8", "ignore")
         if pair is not None and pair != priceFeedKey:
             continue
-        
+
         openInterestNotionalCap = amm.functions.getOpenInterestNotionalCap().call()
-        openInterestNotional = clearingHouse.functions.openInterestNotinalMap(addr).call()
+        openInterestNotional = clearingHouse.functions.openInterestNotionalMap(addr).call()
         maxHoldingBaseAsset = amm.functions.getMaxHoldingBaseAsset().call()
         indexPrice = amm.functions.getUnderlyingPrice().call()
         marketPrice = amm.functions.getSpotPrice().call()
@@ -39,7 +46,7 @@ def getAmmInfo(provider,pair=None):
         baseAssetReserve = reserve[1]
         priceFeed = amm.functions.priceFeed().call()
         estFundingRate = (estimatedFundingRate(amm)/1e18) * 100
-        
+
         if priceFeed == meta.getL2ContractAddress("L2PriceFeed"):
             priceFeedName = "L2PriceFeed"
         elif priceFeed == meta.getL2ContractAddress("ChainlinkPriceFeed"):
@@ -48,17 +55,17 @@ def getAmmInfo(provider,pair=None):
             ValueError("PriceFeed is not L2PriceFeed or ChainlinkPriceFeed, check it immediately!! address: " + priceFeed)
 
         data.append({
-            "pair":f"{priceFeedKey}/USDC",
+            "pair": f"{priceFeedKey}/USDC",
             "Proxy Address": addr,
-            "Index Price": f"{formatUnits(indexPrice)} USDC",
-            "Market Price": f"{formatUnits(marketPrice)} USDC",
-            "OpenInterestNotionalCap": f"{formatUnits(openInterestNotionalCap)} USDC",
-            "OpenInterestNotional": f"{formatUnits(openInterestNotinal)} USDC",
-            "MaxHoldingBaseAsset": f"{formatUnits(maxHoldingBaseAsset)} USDC",
-            "QuoteAssetReserve": f"{formatUnits(quoteAssetReserve)} USDC",
-            "BaseAssetReserve": f"{formatUnits(baseAssetReserve)} {priceFeedKey}USDC",
+            "Index Price": f"{formatUnits(indexPrice[0])} USDC",
+            "Market Price": f"{formatUnits(marketPrice[0])} USDC",
+            "OpenInterestNotionalCap": f"{formatUnits(openInterestNotionalCap[0])} USDC",
+            "OpenInterestNotional": f"{formatUnits(openInterestNotional)} USDC",
+            "MaxHoldingBaseAsset": f"{formatUnits(maxHoldingBaseAsset[0])} USDC",
+            "QuoteAssetReserve": f"{formatUnits(quoteAssetReserve[0])} USDC",
+            "BaseAssetReserve": f"{formatUnits(baseAssetReserve[0])} {priceFeedKey}USDC",
             "PriceFeed": priceFeedName,
             "est.funding rate": f"{estFundingRate} %"
         })
 
-    return data
\ No newline at end of file
+    return data
